@@ -1,11 +1,12 @@
+package KinoSearch::Highlight::Encoder;
 use strict;
 use warnings;
-
-package KinoSearch::Highlight::Encoder;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
-our %instance_vars = ();
+BEGIN {
+    __PACKAGE__->init_instance_vars();
+}
 
 sub encode { shift->abstract_death }
 
@@ -15,7 +16,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Highlight::Encoder - Encode excerpted text.
+KinoSearch::Highlight::Encoder - encode excerpted text
 
 =head1 SYNOPSIS
 
@@ -25,7 +26,7 @@ KinoSearch::Highlight::Encoder - Encode excerpted text.
 
 Encoder objects are invoked by Highlighter objects for every piece of text
 that makes it into an excerpt.  The archetypal implementation is
-L<KinoSearch::Highlight::SimpleHTMLEncoder>.
+KinoSearch::Highlight::SimpleHTMLEncoder.
 
 =head1 METHODS
 
@@ -36,10 +37,10 @@ L<KinoSearch::Highlight::SimpleHTMLEncoder>.
 
 =head1 COPYRIGHT
 
-Copyright 2006-2007 Marvin Humphrey
+Copyright 2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch> version 0.20.
+See L<KinoSearch|KinoSearch> version 0.12.
 
 =cut
