@@ -1,11 +1,12 @@
+package KinoSearch::Highlight::Formatter;
 use strict;
 use warnings;
-
-package KinoSearch::Highlight::Formatter;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
-our %instance_vars = ();
+BEGIN {
+    __PACKAGE__->init_instance_vars();
+}
 
 sub highlight_term { shift->abstract_death }
 
@@ -15,11 +16,11 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Highlight::Formatter - Format highlighted bits within excerpts.
+KinoSearch::Highlight::Formatter - format highlighted bits within excerpts
 
 =head1 SYNOPSIS
 
-    # abstract base class
+	# abstract base class
 
 =head1 DESCRIPTION
 
@@ -30,17 +31,17 @@ excerpt.  The text might be a single term, or it might be a phrase.
 
 =head2 highlight
 
-    my $highlighted = $formatter->highlight($text);
+	my $highlighted = $formatter->highlight($text);
 
-Abstract method.  Highlight text -- by surrounding it with asterisks, html
-"strong" tags, etc.
+Highlight text by e.g. surrounding it with asterisks, or html "strong" tags,
+etc.
 
 =head1 COPYRIGHT
 
-Copyright 2006-2007 Marvin Humphrey
+Copyright 2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch> version 0.20.
+See L<KinoSearch|KinoSearch> version 0.12.
 
 =cut
